package com.huimv.transmission.controller;

import com.huimv.transmission.entity.form.*;
import com.huimv.transmission.result.R;
import com.huimv.transmission.result.RRException;
import com.huimv.transmission.service.DailyTabulateDataService;
import com.huimv.transmission.service.DataPushService;
import com.huimv.transmission.service.EartagDataService;
import com.huimv.transmission.service.FarmEartagChangeService;
import com.huimv.transmission.utils.Constant;
import com.huimv.transmission.utils.PageUtils;
import com.huimv.transmission.utils.TokenUtil;
import org.apache.commons.codec.binary.Hex;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Calendar;
import java.util.Date;


/**
 * <p>
 * 数据推送
 * </p>
 *
 * @author yinhao
 * @date 2021/7/26 10:19
 */
@CrossOrigin
@RestController
@RequestMapping("/pig")
public class PigDataProcessingController {

    @Autowired
    private DataPushService dataPushService;

    @Autowired
    private EartagDataService eartagDataService;

    @Autowired
    private DailyTabulateDataService dailyTabulateDataService;

    @Autowired
    private FarmEartagChangeService farmEartagChangeService;


    @GetMapping("/getAccessToken")
    public R getAccessToken(@Validated TokenForm tokenForm) throws NoSuchAlgorithmException {

        Long timestamp = tokenForm.getTimestamp();
        try {
            new Date(timestamp);
        } catch (Exception e) {
            throw new RRException("时间戳有误!", 10002);
        }
        MessageDigest md = MessageDigest.getInstance("MD5");
        String data = tokenForm.getRandom() + ";" + timestamp;
        String oneSign = Hex.encodeHexString(md.digest(data.getBytes(StandardCharsets.UTF_8)));
        oneSign = oneSign + "zhejiangxumu";
        String twoSign = Hex.encodeHexString(md.digest(oneSign.getBytes(StandardCharsets.UTF_8)));

        if (!tokenForm.getSign().equals(twoSign)) {
            throw new RRException("签名验证失败!", 10003);
        }

        String token = TokenUtil.sign(null);
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.HOUR_OF_DAY, 24);
        Date expireTime = calendar.getTime();
        return R.ok().put("accessToken", token).put("expireTime", expireTime);
    }

    @GetMapping("/getPigStock")
    public R getPigStock(PigStockForm pigStockForm) {
        PageUtils page = dailyTabulateDataService.getPigStock(pigStockForm);
        return R.ok().put("resultData", page);
    }

    @GetMapping("/getNewPigStock")
    public R getNewPigStock(PigStockForm pigStockForm) {
        PageUtils page = farmEartagChangeService.getChangePigStock(pigStockForm, Constant.NEW_EARTAG_TYPE);
        return R.ok().put("resultData", page);
    }

    @GetMapping("/getOffLinePigStock")
    public R getOffLinePigStock() {
        PageUtils page = dataPushService.getOffLinePigStock(1, 10);
        return R.ok().put("resultData", page);
    }

    @PostMapping("/recive")
    public void receiveData(@RequestBody String body) {
        System.out.println(body);
    }


    @GetMapping("/getEartag")
    public R getEartag(@Validated EartagForm eartagForm) {
        PageUtils page = eartagDataService.getEartag(eartagForm);
        return R.ok().put("resultData", page);
    }

    @GetMapping("/getEartagDetails")
    public R getEartagDetails(@Validated EartagDetailForm eartagDetailForm) {
        PageUtils page = eartagDataService.getEartagDetails(eartagDetailForm);
        return R.ok().put("resultData", page);
    }

    @GetMapping("/getPigStockByDay")
    public R getPigStockByDay(@Validated BaseForm baseForm) {
        PageUtils page = dailyTabulateDataService.getPigStockByDay(baseForm);
        return R.ok().put("resultData", page);
    }
}
